#'RainFARM Stochastic Precipitation Downscaling
#'
#'@author Jost von Hardenberg, ISAC-CNR, \email{j.vonhardenberg@isac.cnr.it}
#'@description This function implements the RainFARM stochastic precipitation downscaling method. Adapted for climate downscaling and including orographic correction. References: D'Onofrio et al. 2014, J of Hydrometeorology 15, 830-843; Rebora et. al 2006, JHM 7, 724
#'Has still to be adapted for s2dverification
#'@import JuliaCall
#'@examples 
#'# julia_setup takes a long time only the first time it is called on a machine
#'
#'library(JuliaCall)
#'julia_setup()
#'julia_library("RainFARM")
#'source("rainfarm.R")
#'
#'nf=8; nens=3;
#'# reads a test file with dimension 8x8 and 100 timesteps
#'r=julia_call("read_netcdf2d","pr_ls.nc","pr", need_return = "R" );
#'pr=r[[1]]; lon_mat=r[[2]]; lat_mat=r[[3]];
#'ww <- array(1., dim=c(8*nf,8*nf));
#'rd=rainfarm(pr,lon_mat,lat_mat,nf,fsmooth=TRUE,fglob=FALSE,weights=ww,nens=2,verbose=TRUE)
#'@export

rainfarm <- function(pr, lon_mat, lat_mat, nf, fglob=FALSE,fsmooth=TRUE,weights=1.,nens=1,verbose=FALSE,slope=0) {

library(JuliaCall)
julia_setup()
julia_library("RainFARM")

#r=julia_call("lon_lat_fine",lon_mat, lat_mat,nf,need_return = "R" );
#lon_f=r[[1]]; lat_f=r[[2]];

if(slope==0){
r=julia_call("fft3d",pr,need_return = "R"  );
fxp=r[[1]]; ftp=r[[2]];
sx=julia_call("fitslopex",fxp,kmin=1,need_return = "R" );
} else {
sx=slope;
}

rd=list();
for (j in 1:nens) {
   rd=c(rd,list(julia_call("rainfarmn", pr, sx, nf, weights ,fglob=fglob,fsmooth=fsmooth,verbose=verbose, need_return = "R")));
}

return(rd)
}
